let whitelist = [];

function setPageActionIcon(tabId, greyIcon = false) {
  const icon = greyIcon ? 'icon-grey' : 'icon';
  browser.pageAction.setIcon({
    tabId,
    path: {
      19: `images/${icon}-19.png`,
      38: `images/${icon}-38.png`,
    },
  });
}

function init(storage) {
  if (storage.whitelist) {
    whitelist = storage.whitelist;
  }

  // console.log(whitelist);

  browser.runtime.onMessage.addListener((message, sender) => {
    if (message.msg === 'content-script-loaded') {
      const inWhitelist = whitelist.includes(message.hostname);
      if (!inWhitelist || sender.frameId !== 0) {
        browser.tabs.sendMessage(sender.tab.id, { msg: 'inject-script' }, { frameId: sender.frameId });
      }
      if (sender.frameId === 0) {
        setPageActionIcon(sender.tab.id, inWhitelist);
        browser.pageAction.show(sender.tab.id);
      }
    }
  });
}

function pauseScripts(hostname, value) {
  browser.tabs.query({}).then((tabs) => {
    let tab;
    for (tab of tabs) {
      const tabHostname = new URL(tab.url).hostname;
      if (tabHostname === hostname) {
        setPageActionIcon(tab.id, value);
        browser.tabs.sendMessage(tab.id, { msg: 'pause-script', value });
      }
    }
  });
}

function onPageActionClick(tab) {
  const { hostname } = new URL(tab.url);
  const inWhitelist = whitelist.includes(hostname);

  if (inWhitelist) {
    whitelist.splice(whitelist.indexOf(hostname), 1);
  } else {
    whitelist.push(hostname);
  }

  pauseScripts(hostname, !inWhitelist);
  browser.storage.local.set({ whitelist });
}

browser.storage.local.get(['whitelist']).then(init).catch(e => console.log(e));

browser.pageAction.onClicked.addListener(onPageActionClick);
