(function inject() {
  let paused = false;

  function setEventProperty(object, property) {
    const descriptor = Object.getOwnPropertyDescriptor(object, property);
    Object.defineProperty(object, property, {
      configurable: true,
      enumerable: true,
      get() {
        if (descriptor && descriptor.get) {
          return descriptor.get.call(this);
        }
        return null;
      },
      set(v) {
        let value;
        if (typeof v === 'function') {
          value = (event) => {
            const returnValue = v.call(this, event);
            if (paused
            || (event.type === 'click' && event.button === 0 && !event.ctrlKey && !event.shiftKey)) {
              return returnValue;
            }
            return undefined;
          };
        } else {
          value = v;
        }
        if (descriptor && descriptor.set) {
          descriptor.set.call(this, value);
        }
      },
    });
  }

  const events = [
    'click',
    'mousedown',
    'mouseup',
    'auxclick',
    'contextmenu',
    'selectstart',
    'dragstart',
    'drag',
    'cut',
    'copy',
  ];

  let eventType;
  for (eventType of events) {
    const eventProperty = `on${eventType}`;
    setEventProperty(window, eventProperty);
    setEventProperty(Document.prototype, eventProperty);
    setEventProperty(HTMLElement.prototype, eventProperty);
  }

  const originalPreventDefault = Event.prototype.preventDefault;

  Object.defineProperty(Event.prototype, 'preventDefault', {
    configurable: true,
    writable: false,
    enumerable: true,
    value() {
      if (paused || !events.includes(this.type)
      || (this.type === 'click' && this.button === 0 && !this.ctrlKey && !this.shiftKey)) {
        originalPreventDefault.call(this);
      }
    },
  });

  window.addEventListener('message', (event) => {
    if (event.source === window && event.data.msg === 'ercc:pause-script') {
      paused = event.data.value;
    }
  });
}());
